// ******************************************************************
// ******               SRL/DSP inferencing Example            ******
// ******************************************************************
//
// File:        srl.v 
// Revision:    1.0
// Project:     Inferencing Lab
// Company:     Xilinx
// Date:        March 12, 2013 
//
//
//---------------------------------------------------------------------  
// Description:
//---------------------------------------------------------------------  
//  This is a highly pipelined multiplier and parity generation
//  circuit.
//
//  Without ASYNC_RESET defined, the example will implement async
//  resets.
//  With ASYNC_RESET defined, resets are removed, allowing the
//  tools to inference into an SRL for the pipeline.
//
//  The design should be build and examined as outlined in the lab 
//   description.
//---------------------------------------------------------------------  
`timescale 1ns / 1ps
`define ASYNC_RESET 1

module srl (
  input             CLK,
  input             SYS_RST_L,
  input [15:0]      DATA1, DATA2,
  input             NEW_DATA,
  output reg        DATA_VALID = 1'b0,
  output reg [35:0] DATA_OUT = 36'h000000000
);

`ifdef ASYNC_RESET
  (* async_reg = "true" *) reg reset_reg = 1'b0;
  (* dont_touch = "yes" *) reg           reset_sync  = 1'b0;
  (* dont_touch = "yes" *) reg [15:0]    data1_reg   = 32'h0;
  (* dont_touch = "yes" *) reg [15:0]    data2_reg   = 32'h0;
  (* dont_touch = "yes" *) reg [15:0]    data1_pipe  = 32'h0;
  (* dont_touch = "yes" *) reg [15:0]    data2_pipe  = 32'h0;
  (* dont_touch = "yes" *) reg [31:0]    mult_data_reg = 32'h0;
  (* dont_touch = "yes" *) reg [31:0]    mult_pipe   = 32'h0;
  (* dont_touch = "yes" *) reg [31:0]    mult_pipe2  = 32'h0;
  (* dont_touch = "yes" *) reg [35:0]    mult_par_reg = 36'h0;
  (* dont_touch = "yes" *) reg [35:0]    mult_par_pipe = 36'h0;
  (* dont_touch = "yes" *) reg [3:0]     parity      = 4'h0;
  (* dont_touch = "yes" *) reg [3:0]     data_valid_delay = 4'h0;

`else
    (* async_reg = "true" *) reg reset_reg = 1'b0;
    reg           reset_sync  = 1'b0;
    reg [15:0]    data1_reg   = 32'h0;
    reg [15:0]    data2_reg   = 32'h0;
    reg [15:0]    data1_pipe  = 32'h0;
    reg [15:0]    data2_pipe  = 32'h0;
    reg [31:0]    mult_data_reg = 32'h0;
    reg [31:0]    mult_pipe   = 32'h0;
    reg [31:0]    mult_pipe2  = 32'h0;
    reg [35:0]    mult_par_reg = 36'h0;
    reg [35:0]    mult_par_pipe = 36'h0;
    reg [3:0]     parity      = 4'h0;
    reg [3:0]     data_valid_delay = 4'h0;
 
 `endif

  // Reset synchronization
  // active lo inversion to native active hi done here
  always @(posedge CLK) begin
    reset_sync <= SYS_RST_L;
    reset_reg  <= ~reset_sync;
  end

`ifdef ASYNC_RESET
  always @(posedge CLK, posedge reset_reg)
    if (reset_reg) begin
      data1_reg <= 16'h0000;
      data2_reg <= 16'h0000;
      DATA_VALID <= 1'b0;
    end 
    else if (NEW_DATA) begin
`else
  // NO_RESET
  always @(posedge CLK)
    if (NEW_DATA) begin
`endif
      data1_reg <= DATA1;
      data2_reg <= DATA2;
      DATA_VALID <= data_valid_delay[3];
    end
      
  // Uses an async reset when a reset is not necessary
`ifdef ASYNC_RESET
  always @(posedge CLK, posedge reset_reg) 
    if (reset_reg) begin
      parity        <= 4'h0;
      data1_pipe    <= 32'h00000000;
      data2_pipe    <= 32'h00000000;
      mult_data_reg <= 32'h00000000;
      mult_pipe     <= 32'h00000000;
      mult_pipe2    <= 32'h00000000;
      mult_par_reg  <= 36'h000000000;
      mult_par_pipe <= 36'h000000000;
      data_valid_delay <= 4'h0;
      DATA_OUT      <= 36'h000000000;
    end else 
`else
  // NO_RESET
  always @(posedge CLK)
`endif
    begin
      data1_pipe <= data1_reg;
      data2_pipe <= data2_reg;
      mult_data_reg <= data1_pipe * data2_pipe;
      mult_pipe <= mult_data_reg;
      parity <= {^mult_pipe[31:24], ^mult_pipe[23:16], 
                 ^mult_pipe[15:8], ^mult_pipe[7:0]};
      mult_pipe2 <= mult_pipe;
      mult_par_reg <= {parity[3], mult_pipe2[31:24],
                       parity[2], mult_pipe2[23:16],
                       parity[1], mult_pipe2[15:8],
                       parity[0], mult_pipe2[7:0]};
      data_valid_delay <= {data_valid_delay[2:0], NEW_DATA};
      mult_par_pipe <= mult_par_reg;
      DATA_OUT <= mult_par_pipe;
    end 

endmodule

//<copyright-disclaimer-start>
//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
